<?php
// This file is part of Moodle - http://moodle.org/
//
// Moodle is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// Moodle is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Moodle.  If not, see <http://www.gnu.org/licenses/>.

/**
 * Privacy Subsystem implementation for local_seenkit.
 *
 * @package   block_seenkit
 * @copyright 2022 onwards Abaco Technology  {@link https://www.abacotechnology.com}
 * @license   http://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
 *
 */

namespace block_seenkit\privacy;

use core_privacy\local\metadata\collection;
use core_privacy\local\request\approved_contextlist;
use core_privacy\local\request\contextlist;
use core_privacy\local\request\transform;
use core_privacy\local\request\writer;
use core_privacy\local\request\helper;
use core_privacy\local\request\approved_userlist;
use core_privacy\local\request\userlist;

use block_seenkit\seenkit\seenkit;

defined('MOODLE_INTERNAL') || die();

global $CFG;

/**
 * Implementation of the privacy subsystem plugin provider for block_seenkit.
 *
 * @copyright 2022 onwards Abaco Technology  {@link https://www.abacotechnology.com}
 * @license    http://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
 */
class provider implements
    \core_privacy\local\metadata\provider,
    \core_privacy\local\request\plugin\provider,
    \core_privacy\local\request\user_preference_provider,
    \core_privacy\local\request\core_userlist_provider {

    /**
     * Return the fields which contain personal data.
     *
     * @param collection $collection a reference to the collection to use to store the metadata.
     * @return collection the updated collection of metadata items.
     */
    public static function get_metadata(collection $collection) : collection {
        $collection->add_database_table(
            'block_seenkit',
            [
                'seenkit_internal_id' => 'privacy:metadata:block_seenkit_table:seenkit_internal_id',
                'userid' => 'privacy:metadata:block_seenkit_table:userid'
            ],
            'privacy:metadata:block_seenkit_table'
        );

        $collection->add_external_location_link(
            'seenkit.com',
            [
                'recording_content' => 'privacy:metadata:block_seenkit:recording_content'
            ],
            'privacy:metadata:block_seenkit'
        );

        return $collection;
    }

    /**
     * Get the list of contexts that contain user information for the specified user.
     *
     * @param int $userid the userid.
     * @return contextlist the list of contexts containing user info for the user.
     */
    public static function get_contexts_for_userid(int $userid) : contextlist {
        global $DB;

        $contextlist = new contextlist();

        $user_records = $DB->count_records('block_seenkit', array("userid" => $userid));

        if($user_records > 0){
          $contextlist->add_system_context();
        }

        return $contextlist;
    }

    /**
     * Get the list of users who have data within a context.
     *
     * @param   userlist    $userlist   The userlist containing the list of users who have data in this context/plugin combination.
     */
    public static function get_users_in_context(userlist $userlist) {
        $context = $userlist->get_context();
        if ($context->contextlevel != CONTEXT_SYSTEM) {
            return;
        }

        $sql = "SELECT DISTINCT userid
                  FROM {block_seenkit}";

        $userlist->add_from_sql('userid', $sql, array());

    }

    /**
     * Export personal data for the given approved_contextlist.
     * User and context information is contained within the contextlist.
     *
     * @param approved_contextlist $contextlist a list of contexts approved for export.
     */
    public static function export_user_data(approved_contextlist $contextlist) {
        global $DB;

        if (empty($contextlist->count())) {
            return;
        }

        $user = $contextlist->get_user();

        foreach ($contextlist->get_contexts() as $context) {
            if ($context->contextlevel != CONTEXT_SYSTEM) {
                continue;
            }

            $block_seenkit = (object) [
                'name' => get_string('privacy:metadata:block_seenkit:seenkit_export', 'block_seenkit')
            ];
            writer::with_context($context)->export_data([], $block_seenkit);

            $user_records = $DB->count_records('block_seenkit', array('userid' => $user->id));
            if($user_records > 0){
              $user_seenkit_internal_ids = $DB->get_fieldset_select('block_seenkit', 'seenkit_internal_id', 'userid = :userid', array('userid' => $user->id));
              $subcontexts = [
                  get_string('privacy:user', 'block_seenkit')
              ];
              $userdata = (object) [
                  'seenkit_internal_id' => $user_seenkit_internal_ids,
                  'user' => transform::user($user->id)
              ];
              writer::with_context($context)
                  ->export_data($subcontexts, $userdata);
            }
        }
    }

    /**
     * Stores the user preferences related to block_seenkit.
     *
     * @param  int $userid The user ID that we want the preferences for.
     */
    public static function export_user_preferences(int $userid) {
        $context = \context_system::instance();
        $seenkitpreferences = []; // No specific user preferences in this pluginin (block_seenkit)

        foreach ($seenkitpreferences as $key => $preference) {
            $value = get_user_preferences($key, null, $userid);
            if ($preference['bool']) {
                $value = transform::yesno($value);
            }
            if (isset($value)) {
                writer::with_context($context)
                    ->export_user_preference('block_seenkit', $key, $value, $preference['string']);
            }
        }
    }

    /**
     * Delete all data for all users in the specified context.
     *
     * @param \context $context the context to delete in.
     */
    public static function delete_data_for_all_users_in_context(\context $context) {
        global $DB, $CFG;

        if ($context->contextlevel == CONTEXT_SYSTEM) {
          $config = get_config('block_seenkit');
          $seenkit = new seenkit($config->api_key, $config->api_secret, $config->api_url, $CFG->wwwroot);
          $recordings = $DB->get_records('block_seenkit');
          foreach($recordings as $recording){
            $del_result = json_decode($seenkit->delete_unit($recording->seenkit_internal_id));
			//$del_result = (object)["res"=>"ok"]; // When testing with PhpUnit we cannot actually delete a recording that does not exist on Seenkit
        	if($del_result->res == "ok"){
              $DB->delete_records_select('block_seenkit', $DB->sql_compare_text('seenkit_internal_id') . ' = ' .$DB->sql_compare_text(':seenkit_internal_id'),  array('seenkit_internal_id' => $recording->seenkit_internal_id));
            }
          }
        }
    }

    /**
     * Delete all user data for the specified user, in the specified contexts.
     *
     * @param approved_contextlist $contextlist a list of contexts approved for deletion.
     */
    public static function delete_data_for_user(approved_contextlist $contextlist) {
        global $DB, $CFG;

        if (empty($contextlist->count())) {
            return;
        }

        $userid = $contextlist->get_user()->id;
        $contexts = $contextlist->get_contexts();
        foreach($contexts as $context){
          if ($context->contextlevel == CONTEXT_SYSTEM) {
             $config = get_config('block_seenkit');
             $seenkit = new seenkit($config->api_key, $config->api_secret, $config->api_url, $CFG->wwwroot);
             $recordings = $DB->get_records('block_seenkit', array('userid' => $userid));
             foreach ($recordings as $recording) {
                $del_result = json_decode($seenkit->delete_unit($recording->seenkit_internal_id));
				//$del_result = (object)["res"=>"ok"]; // When testing with PhpUnit we cannot actually delete a recording that does not exist on Seenkit
           		if($del_result->res == "ok"){
                  $DB->delete_records_select('block_seenkit', $DB->sql_compare_text('seenkit_internal_id') . ' = ' .$DB->sql_compare_text(':seenkit_internal_id'),  array('seenkit_internal_id' => $recording->seenkit_internal_id));
                }
             }
           }
        }
    }

    /**
     * Delete multiple users within a single context.
     *
     * @param   approved_userlist       $userlist The approved context and user information to delete information for.
     */
    public static function delete_data_for_users(approved_userlist $userlist) {
        global $DB, $CFG;

        $context = $userlist->get_context();
        if ($context->contextlevel != CONTEXT_SYSTEM) {
            return;
        }

        $config = get_config('block_seenkit');
        $seenkit = new seenkit($config->api_key, $config->api_secret, $config->api_url, $CFG->wwwroot);
        $userids = $userlist->get_userids();
        list($insql, $inparams) = $DB->get_in_or_equal($userids);
        $sql = "SELECT * FROM {block_seenkit} WHERE userid $insql";
        $recordings = $DB->get_records_sql($sql, $inparams);
        foreach ($recordings as $recording) {
            $del_result = json_decode($seenkit->delete_unit($recording->seenkit_internal_id));
			//$del_result = (object)["res"=>"ok"]; // When testing with PhpUnit we cannot actually delete a recording that does not exist on Seenkit
            if($del_result->res == "ok"){
              $DB->delete_records_select('block_seenkit', $DB->sql_compare_text('seenkit_internal_id') . ' = ' .$DB->sql_compare_text(':seenkit_internal_id'),  array('seenkit_internal_id' => $recording->seenkit_internal_id));
            }
        }
    }

}
