<?php
// This file is part of Moodle - http://moodle.org/
//
// Moodle is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// Moodle is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Moodle.  If not, see <http://www.gnu.org/licenses/>.

/**
 * Privacy test for seenkit plugin
 *
 * @package   mod_seenkit
 * @copyright 2020 onwards Abaco Technology  {@link https://www.abacotechnology.com}
 * @license   http://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
 */

defined('MOODLE_INTERNAL') || die();

require_once(__DIR__ . '/../lib.php');

use \mod_seenkit\privacy\provider;
use \core_privacy\local\request\approved_contextlist;
use \core_privacy\local\request\transform;
use \core_privacy\local\request\writer;
use \core_privacy\tests\provider_testcase;

/**
 * rivacy test for seenkit plugin
 *
 * @package    mod_seenkit
 * @copyright  2020 onwards Abaco Technology  {@link https://www.abacotechnology.com}
 * @license    http://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
 */
//class mod_seenkitengage_privacy_testcase extends provider_testcase {
class privacy_test extends provider_testcase {

    /**
     * @var array   Array of teachers.
     */
    private $courseteachers = [];

    /**
     * @var array   Array of courses.
     */
    private $courses = [];

    /**
     * @var array   Array of test seenkit associated for Course 1.
     */
    private $course1seenkits = [];

    /**
     * @var array   Array of test seenkit associated for Course 2.
     */
    private $course2seenkits = [];


    /**
     * Test for provider::get_contexts_for_userid().
     *
     * @throws coding_exception
     */

    public function test_get_contexts_for_userid() {
        global $DB;

        $this->resetAfterTest(true);
        $this->create_courses_and_seenkits();
        $contextids = provider::get_contexts_for_userid($this->courseteachers['t1']->id);

        $this->assertEquals(2, count($contextids->get_contextids()));

    }


    /**
     * Test that the correct userids are returned for a specific context.
     */

    public function test_get_users_in_context() {

        $this->resetAfterTest(true);
        $this->create_courses_and_seenkits();

        // Try with course 1 seenkit 1 context.
        $c1s1ctx = context_module::instance($this->course1seenkits['s1']->cmid);
        $userlist = new \core_privacy\local\request\userlist($c1s1ctx, 'mod_seenkit');
        provider::get_users_in_context($userlist);
        $userids = $userlist->get_userids();
        $this->assertCount(1, $userids);
        $this->assertFalse(in_array($this->courseteachers['t2']->id, $userids));

        // Try with the course context.
        $coursecontext = context_course::instance($this->courses['c1']->id);
        $userlist = new \core_privacy\local\request\userlist($coursecontext, 'mod_seenkit');
        provider::get_users_in_context($userlist);
        $this->assertEmpty($userlist->get_userids());
    }


    /**
     * Test for provider::export_user_data().
     *
     * @throws dml_exception
     */

    public function test_export_user_data() {
        global $DB;

        $this->resetAfterTest(true);
        $this->create_courses_and_seenkits();

        $teacher1 = $this->courseteachers['t1'];

        // Verify the contexts
        $contextlist = provider::get_contexts_for_userid($teacher1->id);
        $contexts = $contextlist->get_contexts();
        $context = context_module::instance($this->course2seenkits['s2']->cmid);
        $this->assertContains($context, $contexts);

        // Verify exported data
        $approvedcontextlist = new approved_contextlist($teacher1, 'mod_seenkit', $contextlist->get_contextids());
        provider::export_user_data($approvedcontextlist);
        $writer = writer::with_context($context);
        $subcontexts = [
            get_string('privacy:ownerpath', 'mod_seenkit')
        ];
        $seenkit = $writer->get_data($subcontexts);
        $this->assertEquals("12345679123456789-course2s2teacher1", $seenkit->seenkit_internal_id);
    }

    /**
     * Test for provider::delete_data_for_all_users_in_context().
     *
     * @throws dml_exception
     */

    public function test_delete_data_for_all_users_in_context() {
        global $DB;

        $this->resetAfterTest(true);
        $this->create_courses_and_seenkits();

        // Test teacher1 delete_data_for_all_users_in_context().
        $teacher1 = $this->courseteachers['t1'];
        $contextlist = provider::get_contexts_for_userid($teacher1->id);

        foreach ($contextlist as $context) {
            provider::delete_data_for_all_users_in_context($context);
        }

        $c1s1ctx = context_module::instance($this->course2seenkits['s2']->cmid);
        $userlist = new \core_privacy\local\request\userlist($c1s1ctx, 'mod_seenkit');
        provider::get_users_in_context($userlist);
        $this->assertEmpty($userlist->get_userids());
    }

    /**
     * Test for provider::delete_data_for_user().
     *
     * @throws dml_exception
     */

    public function test_delete_data_for_user() {
        global $DB;

        $this->resetAfterTest(true);
        $this->create_courses_and_seenkits();

        $teacher1 = $this->courseteachers['t1'];
        $contextlist = provider::get_contexts_for_userid($teacher1->id);
        $approvedcontextlist = new approved_contextlist($teacher1, 'mod_seenkit', $contextlist->get_contextids());
        provider::delete_data_for_user($approvedcontextlist);

        $contextlist = provider::get_contexts_for_userid($teacher1->id);
        $this->assertEmpty($contextlist->get_contextids());
    }

    /**
     * Test the deletion of a data for users.
     */

    public function test_delete_data_for_users() {
        global $DB;

        $this->resetAfterTest(true);
        $this->create_courses_and_seenkits();


        $context = context_module::instance($this->course1seenkits['s1']->cmid);

        $userlist = new \core_privacy\local\request\approved_userlist($context, 'mod_seenkit', [$this->courseteachers['t1']->id, $this->courseteachers['t2']->id]);
        provider::delete_data_for_users($userlist);

        $contextlist = provider::get_contexts_for_userid($this->courseteachers['t1']->id);
        $this->assertCount(1, $contextlist->get_contextids());
    }

    // Start of helper functions.

    /**
     * Helper function to setup Course, users, and assignments for testing.
     */

    protected function create_courses_and_seenkits() {
        // Create Courses, Users, and Seenkit modules.
        $course1 = $this->getDataGenerator()->create_course(['shortname' => 'course1']);
        $course2 = $this->getDataGenerator()->create_course(['shortname' => 'course2']);

        $this->courses = [
          'c1' => $course1,
          'c2' => $course2
        ];

        $teacher1 = $this->getDataGenerator()->create_user(['username' => 'teacher1']);
        $teacher2 = $this->getDataGenerator()->create_user(['username' => 'teacher2']);

        $this->courseteachers = [
            't1' => $teacher1,
            't2' => $teacher2,
        ];

        $course1seenkit1 = $this->getDataGenerator()->create_module('seenkit',
            [
                'course' => $course1->id,
                'name' => 'Course 1 - Seenkit 1 (recorded lesson)',
                'recording_owner' => $teacher1->id,
                'seenkit_internal_id' => '12345679123456789-course1s1teacher1'
            ]
        );

        $this->course1seenkits = [
            's1' => $course1seenkit1,
        ];

        $course2seenkit1 = $this->getDataGenerator()->create_module('seenkit',
            [
                'course' => $course2->id,
                'name' => 'Course 2 - Seenkit 1 (recorded lesson)',
                'recording_owner' => $teacher2->id,
                'seenkit_internal_id' => '12345679123456789-course2s1teacher2'
            ]
        );


        $course2seenkit2 = $this->getDataGenerator()->create_module('seenkit',
            [
                'course' => $course2->id,
                'name' => 'Course 2 - Seenkit 2 (recorded lesson)',
                'recording_owner' => $teacher1->id,
                'seenkit_internal_id' => '12345679123456789-course2s2teacher1'
            ]
        );

        $this->course2seenkits = [
            's1' => $course2seenkit1,
            's2' => $course2seenkit2
        ];

    }
}
