<?php
// This file is part of Moodle - http://moodle.org/
//
// Moodle is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// Moodle is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Moodle.  If not, see <http://www.gnu.org/licenses/>.

/**
 * Privacy test for block_seenkit plugin
 *
 * @package   block_seenkit
 * @copyright 2022 onwards Abaco Technology  {@link https://www.abacotechnology.com}
 * @license   http://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
 */

defined('MOODLE_INTERNAL') || die();

use \block_seenkit\privacy\provider;
use \core_privacy\local\request\approved_contextlist;
use \core_privacy\local\request\transform;
use \core_privacy\local\request\writer;
use \core_privacy\tests\provider_testcase;

/**
 * Privacy test for block_seenkit plugin
 *
 * @package    block_seenkit
 * @copyright  2022 onwards Abaco Technology  {@link https://www.abacotechnology.com}
 * @license    http://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
 */

class privacy_test extends provider_testcase {

    /**
     * @var array   Array of teachers.
     */
    private $users = [];



    /**
     * Test for provider::get_contexts_for_userid().
     *
     * @throws coding_exception
     */

    public function test_get_contexts_for_userid() {
        global $DB;

        $this->resetAfterTest(true);
        $this->create_users_and_recordings();
        $contextids = provider::get_contexts_for_userid($this->users['u1']->id);

        $this->assertEquals(1, count($contextids->get_contextids()));
    }


    /**
     * Test that the correct userids are returned for a specific context.
     */

    public function test_get_users_in_context() {

        $this->resetAfterTest(true);
        $this->create_users_and_recordings();

        $ctx = context_system::instance();
        $userlist = new \core_privacy\local\request\userlist($ctx, 'block_seenkit');
        provider::get_users_in_context($userlist);
        $userids = $userlist->get_userids();
        $this->assertCount(2, $userids);
    }


    /**
     * Test for provider::export_user_data().
     *
     * @throws dml_exception
     */

    public function test_export_user_data() {
        global $DB;

        $this->resetAfterTest(true);
        $this->create_users_and_recordings();

        $user1 = $this->users['u1'];

        // Verify the contexts
        $contextlist = provider::get_contexts_for_userid($user1->id);
        $contexts = $contextlist->get_contexts();
        $context = context_system::instance();
        $this->assertContains($context, $contexts);

        // Verify exported data
        $approvedcontextlist = new approved_contextlist($user1, 'block_seenkit', $contextlist->get_contextids());
        provider::export_user_data($approvedcontextlist);
        $writer = writer::with_context($context);

        $subcontexts = [
            get_string('privacy:user', 'block_seenkit')
        ];
        $user = $writer->get_data($subcontexts);
        $this->assertEquals(2, count($user->seenkit_internal_id));

    }

    /**
     * Test for provider::delete_data_for_all_users_in_context().
     *
     * @throws dml_exception
     */

    public function test_delete_data_for_all_users_in_context() {
        global $DB;

        $this->resetAfterTest(true);
        $this->create_users_and_recordings();

        $ctx = context_system::instance();
        provider::delete_data_for_all_users_in_context($ctx);

        $userlist = new \core_privacy\local\request\userlist($ctx, 'block_seenkit');
        provider::get_users_in_context($userlist);
        $this->assertEmpty($userlist->get_userids());
    }

    /**
     * Test for provider::delete_data_for_user().
     *
     * @throws dml_exception
     */

    public function test_delete_data_for_user() {
        global $DB;

        $this->resetAfterTest(true);
        $this->create_users_and_recordings();

        $user1 = $this->users['u1'];
        $contextlist = provider::get_contexts_for_userid($user1->id);
        $approvedcontextlist = new approved_contextlist($user1, 'block_seenkit', $contextlist->get_contextids());
        provider::delete_data_for_user($approvedcontextlist);

        $contextlist = provider::get_contexts_for_userid($user1->id);
        $this->assertEmpty($contextlist->get_contextids());
    }

    /**
     * Test the deletion of a data for users.
     */

    public function test_delete_data_for_users() {
        global $DB;

        $this->resetAfterTest(true);
        $this->create_users_and_recordings();


        $context = context_system::instance();

        $userlist = new \core_privacy\local\request\approved_userlist($context, 'block_seenkit', [$this->users['u1']->id, $this->users['u2']->id]);
        provider::delete_data_for_users($userlist);

        $contextlist = provider::get_contexts_for_userid($this->users['u1']->id);
        $this->assertEmpty($contextlist->get_contextids());
    }

    // Start of helper functions.

    /**
     * Helper function to setup Course, users, and assignments for testing.
     */

    protected function create_users_and_recordings() {
        global $DB;

        $user1 = $this->getDataGenerator()->create_user(['username' => 'user1']);
        $user2 = $this->getDataGenerator()->create_user(['username' => 'user2']);

        $this->users = [
            'u1' => $user1,
            'u2' => $user2,
        ];

        $u1recording1 = new \stdClass();
        $u1recording1->userid = $user1->id;
        $u1recording1->seenkit_internal_id = '12345679123456789-u1rec1';
        $u1recording1->timecreated = time();
        $DB->insert_record('block_seenkit', $u1recording1);

        $u1recording2 = new \stdClass();
        $u1recording2->userid = $user1->id;
        $u1recording2->seenkit_internal_id = '12345679123456789-u1rec2';
        $u1recording2->timecreated = time();
        $DB->insert_record('block_seenkit', $u1recording2);

        $u2recording1 = new \stdClass();
        $u2recording1->userid = $user2->id;
        $u2recording1->seenkit_internal_id = '12345679123456789-u2rec1';
        $u2recording1->timecreated = time();
        $DB->insert_record('block_seenkit', $u2recording1);
    }
}
